/* LISTDEMO.CPP - Beispiel aus Kapitel 5 der
   Einfhrung */

// Demonstriert dynamische Objekte

// mit FIGURES.OBJ und GRAPHICS.LIB linken

#include <conio.h>          // fr getch()
#include <alloc.h>          // fr coreleft()
#include <stdlib.h>         // fr itoa()
#include <string.h>         // fr strcpy()
#include <graphics.h>
#include "figures.h"

class Arc : public Circle
{
   int StartAngle, EndAngle;
public:

   // Konstruktor
   Arc(int InitX, int InitY, int InitRadius,
       int InitStartAngle, int InitEndAngle);

   // Virtuelle Funktionen
   void Show();
   void Hide();
};

struct Node
{     
   Point *Item;  // Der Listeneintrag kann Point
                 //  oder jede von Point
		 //  abgeleitete Klasse sein
   Node  *Next;  // zeigt auf das nchste
                 //  Node-Objekt (einen Knoten)
};

class List  // Die Objekt-Liste, auf die die
            //  Knoten zeigen
{
   Node *Nodes;  // Zeigt auf einen Knoten
public:

   // Konstruktor
   List();

   // Destruktor
   ~List();

   // Einen Eintrag in die Liste vornehmen
   void Add(Point *NewItem);

   // Die Eintrge auflisten
   void Report();
};

// Definitionen fr alleinstehende Funktionen

void OutTextLn(char *TheText)
{
   outtext(TheText);
   moveto(0, gety() + 12); // Zur nchsten Zeile
}

void MemStatus(char *StatusMessage)
{
   // fr den Rckgabewert von coreleft()
   unsigned long MemLeft;

   // temporrer String, wird an outtext()
   //  geschickt
   char CharString[12];
   outtext(StatusMessage);

   MemLeft = long (coreleft());

   // Ergebnis mit ltoa in String umwandeln, 
   //  dann in einen temporren String kopieren
   ltoa(MemLeft, CharString, 10);
   OutTextLn(CharString);
}

// Elementfunktionen der Klasse Arc

Arc::Arc(int InitX, int InitY, int InitRadius,
         int InitStartAngle, int InitEndAngle)
    : Circle (InitX, InitY,InitRadius)
      // ruft den Circle-Konstruktor auf
{
   StartAngle = InitStartAngle;
   EndAngle = InitEndAngle;
}

void Arc::Show()
{
   Visible = true;
   arc(X, Y, StartAngle, EndAngle, Radius);
}

void Arc::Hide()
{
   unsigned TempColor;
   TempColor = getcolor();
   setcolor(getbkcolor());
   Visible = false;
   arc(X, Y, StartAngle, EndAngle, Radius);
   setcolor(TempColor);
}

// Elementfunktionen der Klasse List

List::List () {
   Node *N;
   N = new Node;
   N->Item = NULL;
   N->Next = NULL;
   Nodes = NULL;  // Knotenzeiger auf "Null"
		  //  setzen, da die Liste leer ist
}

List::~List()     // Destruktor
{
   // bis zum Ende der Liste
   while (Nodes != NULL)
   {
      // Zeiger auf einen Knoten, dessen
      //  Inhalt gelscht wird
      Node *N = Nodes;
      delete(N->Item);

      // zeigt auf den nchsten Knoten       
      Nodes = N->Next;

      // den Speicher des Zeigers lschen
      delete N;
   };
}

void List::Add(Point *NewItem)
{
   // N zeigt auf einen Knoten
   Node *N;

   // einen neuen Knoten erzeugen
   N = new Node;

   // Zeiger auf das Objekt im Knoten speichern
   N->Item = NewItem;

   // nchster Eintrag zeigt auf akt. Listenposition
   N->Next = Nodes;

   // letzter Eintrag zeigt nun auf diesen Knoten
   Nodes = N;

}

void List::Report()
{
   char TempString[12];
   Node *Current = Nodes;
   while (Current != NULL)
   {
      // X-Wert des Eintrags des aktuellen Knotens
      //  lesen und in einen String konvertieren
      itoa(Current->Item->GetX(), TempString, 10);
      outtext("X = ");
      OutTextLn(TempString);

      // das gleiche fr den Y-Wert
      itoa(Current->Item->GetY(), TempString, 10);
      outtext("Y = ");
      OutTextLn(TempString);

      // auf den nchsten Knoten zeigen
      Current = Current->Next;
   };
}

void setlist(void);

main()     // Funktionen testen
{
   int graphdriver = DETECT, graphmode;
   initgraph(&graphdriver, &graphmode,
             "..\\bgi");

   MemStatus("Speicher freigeben, bevor die "
             "Liste initialisiert wird: ");
   setlist();
   MemStatus("Speicher freigeben nach "
             "List-Destruktor: ");
   getch();
   closegraph();
}

void setlist() {

   // eine Liste deklarieren
   //  (ruft den List-Konstruktor auf)
   List AList;

   // einige Figuren erzeugen und in die
   //  Liste aufnehmen
   Arc *Arc1 = new Arc(151, 82, 25, 200, 330);
   AList.Add(Arc1);
   MemStatus("Speicher freigeben nach Zufgen "
             "von arc1: ");

   Circle *Circle1 = new Circle(200, 80, 40);
   AList.Add(Circle1);
   MemStatus("Speicher freigeben nach Zufgen "
             "von circle1: ");

   Circle *Circle2 = new Circle(305, 136, 35);
   AList.Add(Circle2);
   MemStatus("Speicher freigeben nach Zufgen "
             "von circle2: ");

   // Liste durchgehen und X-Y-Werte der
   //  Figuren zeigen
   AList.Report();

   /* Die 3 Alist-Knoten und die Objekte Arc und
      Circle werden von ihren Destruktoren
      automatisch freigegeben, wenn sie nicht
      mehr im Gltigkeitsbereich von main() sind.
      Arc und Circle benutzen implizit Destruktoren
      im Gegensatz zum expliziten ~List-Destruktor.
      Sie knnen hier aber auch explizit lschen,
      wenn Sie das wnschen:

      delete Arc1; delete Circle1; delete Circle2;
   */

   getch();   // auf einen Tastendruck warten
   return;
}