/*------------------------------------------------------------------------
 * Dateiname - matherr.c
 *
 * Funktion    matherr - Behandlung mathematischer Fehler
 *-----------------------------------------------------------------------*/

/*[]---------------------------------------------------[]*/
/*|                                                     |*/
/*|    Turbo C Runtime-Bibliothek                       |*/
/*|                                                     |*/
/*|                                                     |*/
/*|    Copyright (c) 1987,1990 by Borland International |*/
/*|    Alle Rechte vorbehalten.                         |*/
/*|                                                     |*/
/*[]---------------------------------------------------[]*/


#include <math.h>

#ifdef  UNIX_matherr
#include <stdio.h>
#include <process.h>

char *whyS [] =
{
    "argument domain error",
    "argument singularity ",
    "overflow range error ",
    "underflow range error",
    "total loss of significance",
    "partial loss of significance"
};

/*------------------------------------------------------------------------*

Name            matherr - Behandlung mathematischer Fehler

Gebrauch        include <math.h>
                int matherr(struct exception *e);

Prototyp in     math.h

Beschreibung    Wenn in der Mathematik-Bibliothek Probleme entdeckt werden,
                wird _matherr() mit allen verfgbaren Informationen aufge-
                rufen.

                Die Funktion fhrt relativ wenig aus, sie schreibt die
                Ursache des Problems (ERANGE oder EDOMAIN) in errno.
                Sie dient hauptschlich als Mittelpunkt fr nderungen
                in der Fehlerbehandlung.

                Wenn Sie zum Beispiel eine Tabellenkalkulation schreiben,
                knnten Sie diese Funktion durch eine andere ersetzen, die
                eine Fehlermeldung in einem Fenster zeigt:

                        "log (-2.0) caused domain error, in cell J7"

                und dann mit longjmp() in einen Reset-Zustand springen um
                den nchsten Befehl des Anwenders zu warten.

                Die Standardversion der Turbo C-matherr-Routine meldet
                keine Underflow- und Precision-Fehler; andere Fehler werden
                als schwerwiegend angesehen. matherr dient als Grundlage,
                auf der Sie aufsetzen sollten, wenn Sie Ihre eigene
                Fehlerbehandlungsroutine schreiben.

                Der Grund fr das Nichtmelden der Underflow- und Precision-
                Fehler ist, da diese Fehler nicht in ANSI C spezifiziert
                sind. Daher bekommen Sie
                        exp(-1000) = 0
                        sin(1e100) = NAN
                ohne Fehler oder Warnung, auch wenn in beiden Fllen die
                gesamte Rechengenauigkeit verloren geht. Solche Probleme
                knnen Sie durch Ihre eigene matherr-Routine abfangen.

                Mgliche Fehler sind DOMAIN, SING, OVERFLOW, UNDERFLOW,
                TLOSS, PLOSS (in <math.h> aufgefhrt). Wie schon oben erklrt,
                werden UNDERFLOW und TLOSS von der Standard-matherr behandelt.
                PLOSS wird von Turbo C nicht untersttzt und wird von
                keiner Bibliotheksfunktion erzeugt.

                Sie knnen matherr dahingehend modifizieren, da sie zu
                einer an Ihre Anwendungen angepaten Fehler-Routine wird.
                Sie sollte 0 zurckgeben, wenn es nicht mglich war, einen
                Fehler zu beheben, und einen Wert ungleich Null, wenn ein
                Fehler korrigiert worden ist. Wenn matherr 0 zurckgibt, wird
                weder errno gendert noch eine Fehlermeldung ausgegeben.

                Wir wissen natrlich nicht, welche Fehlerbehandlung Sie
                mchten, aber Sie knnen sicher sein, da alle Fehler mit
                allen zur Verfgung stehenden Informationen bei matherr() an-
                kommen, so da Sie Ihre Lsungen frei gestalten knnen.

                In Turbo C ist nicht die Funktion matherr() integriert, die
                UNIX-Benutzer kennen, da der ANSI x3j11 Draft eine inkompa-
                tible Version untersttzt. Die Version, die Sie erhalten, ist
                so kompatibel wie mglich, ohne die ANSI-Regeln zu brechen.
                Natrlich knnen Sie sie auch in eine UNIX-Version kon-
                vertieren. Der notwendige Code ist bereits vorhanden, Sie
                mssen ihn nur noch aktivieren.

Rckgabewert    Der Standard-Rckgabewert von matherr ist 0. Matherr kann
                aber auch e->retval verwenden, was das Ergebnis ber
                _matherr zum Aufrufenden zurckgibt.

                Wenn matherr 0 zurckgibt (und damit meldet, da der Fehler
                nicht behoben werden konnte), wird eine Fehlermeldung ausge-
                geben und Errno ein Wert zugewiesen.

                Wenn matherr einen Wert ungleich Null zurckgibt (und damit
                meldet, da der Fehler behoben werden konnte), wird weder
                eine Meldung ausgegeben noch errno ein Wert zugewiesen.

*-------------------------------------------------------------------------*/
int matherr (struct exception *e)
{
    fprintf (stderr,
        "%s (%8g,%8g): %s\n", e->name, e->arg1, e->arg2, whyS [e->type - 1]);

    exit (1);
}
#else

int matherr(struct exception *e)
{
        if (e->type == UNDERFLOW)
        {
                e->retval = 0;
                return 1;
        }
        if (e->type == TLOSS)
        {
                /* Genauigkeitsverlust, doch das Problem wird ignoriert */
                return 1;
        }
        /* alle anderen Fehler sind fatal */
        return 0;
}


#endif
